@@ -1,28 +1,52 @@
 <?php
 
-// Sort types
-/** Sort by task name. */
+/**
+ * Sort by task name.
+ */
 define('NAG_SORT_NAME', 'name');
-/** Sort by priority. */
+
+/**
+ * Sort by priority.
+ */
 define('NAG_SORT_PRIORITY', 'priority');
-/** Sort by due date. */
+
+/**
+ * Sort by due date.
+ */
 define('NAG_SORT_DUE', 'due');
-/** Sort by completion. */
+
+/**
+ * Sort by completion.
+ */
 define('NAG_SORT_COMPLETION', 'completed');
-/** Sort by category. */
+
+/**
+ * Sort by category.
+ */
 define('NAG_SORT_CATEGORY', 'category');
-/** Sort by owner. */
+
+/**
+ * Sort by owner.
+ */
 define('NAG_SORT_OWNER', 'tasklist_id');
 
-/** Sort in ascending order. */
+/**
+ * Sort in ascending order.
+ */
 define('NAG_SORT_ASCEND', 0);
-/** Sort in descending order. */
+
+/**
+ * Sort in descending order.
+ */
 define('NAG_SORT_DESCEND', 1);
 
 /**
  * Nag Base Class.
  *
- * $Horde: nag/lib/Nag.php,v 1.124.2.8 2005/10/10 01:41:20 selsky Exp $
+ * $Horde: nag/lib/Nag.php,v 1.124.2.17 2006/08/03 13:33:45 jan Exp $
+ *
+ * See the enclosed file COPYING for license information (GPL).  If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
  * @package Nag
@@ -72,6 +96,11 @@ class Nag {
      * @param constant $sortdir    The direction by which to sort
      *                             (NAG_SORT_ASC, NAG_SORT_DESC).
      * @param constant $altsortby  The secondary sort field.
+     * @param array $tasklists     An array of tasklist to display or
+     *                             null/empty to display taskslists
+     *                             $GLOBALS['display_tasklists'].
+     * @param integer $completed   Which tasks to retrieve (1 = all tasks,
+     *                             0 = incomplete tasks, 2 = complete tasks).
      *
      * @return array  A list of the requested tasks.
      *
@@ -79,7 +108,9 @@ class Nag {
      */
     function listTasks($sortby = null,
                        $sortdir = null,
-                       $altsortby = null)
+                       $altsortby = null,
+                       $tasklists = null,
+                       $completed = null)
     {
         global $prefs, $registry;
 
@@ -92,7 +123,17 @@ class Nag {
         if (is_null($altsortby)) {
             $altsortby = $prefs->getValue('altsortby');
         }
+        if ($completed === null) {
+            $completed = $prefs->getValue('show_completed');
+        }
 
+        if (is_null($tasklists)) {
+            $tasklists = $GLOBALS['display_tasklists'];
+        }
+        if (!is_array($tasklists)) {
+            $tasklists = array($tasklists);
+        }
+        
         /* Sorting criteria for the task list. */
         $sort_functions = array(
             NAG_SORT_PRIORITY => 'ByPriority',
@@ -104,10 +145,12 @@ class Nag {
         );
 
         $tasks = array();
-        foreach ($GLOBALS['display_tasklists'] as $tasklist) {
+        $view_url = Horde::applicationUrl('view.php');
+        $task_url = Horde::applicationUrl('task.php');
+        foreach ($tasklists as $tasklist) {
             /* Create a Nag storage instance. */
             $storage = &Nag_Driver::singleton($tasklist);
-            $storage->retrieve();
+            $storage->retrieve($completed);
 
             /* Retrieve the tasklist from storage. */
             $newtasks = $storage->listTasks();
@@ -115,19 +158,15 @@ class Nag {
                 return $newtasks;
             }
 
+            $view_url_list = Util::addParameter($view_url, 'tasklist', $tasklist);
+            $task_url_list = Util::addParameter($task_url, 'tasklist', $tasklist);
             foreach ($newtasks as $taskID => $task) {
-                $url = Util::addParameter('view.php',
-                                          array('task' => $task['task_id'],
-                                                'tasklist' => $task['tasklist_id']));
-                $newtasks[$taskID]['view_link'] = Horde::applicationUrl($url);
+                $newtasks[$taskID]['view_link'] = Util::addParameter($view_url_list, 'task', $task['task_id']);
 
-                $url = Util::addParameter('task.php',
-                                          array('task' => $task['task_id'],
-                                                'tasklist' => $task['tasklist_id']));
-                $url = Horde::applicationUrl($url);
-                $newtasks[$taskID]['complete_link'] = Util::addParameter($url, 'actionID', 'complete_task');
-                $newtasks[$taskID]['edit_link'] = Util::addParameter($url, 'actionID', 'modify_task');
-                $newtasks[$taskID]['delete_link'] = Util::addParameter($url, 'actionID', 'delete_tasks');
+                $task_url_task = Util::addParameter($task_url_list, 'task', $task['task_id']);
+                $newtasks[$taskID]['complete_link'] = Util::addParameter($task_url_task, 'actionID', 'complete_task');
+                $newtasks[$taskID]['edit_link'] = Util::addParameter($task_url_task, 'actionID', 'modify_task');
+                $newtasks[$taskID]['delete_link'] = Util::addParameter($task_url_task, 'actionID', 'delete_tasks');
             }
 
             $tasks = array_merge($tasks, $newtasks);
@@ -152,26 +191,6 @@ class Nag {
             }
         }
 
-        /* Filter complete/incomplete tasks if the user doesn't want them
-         * shown. */
-        switch ($prefs->getValue('show_completed')) {
-        case 0:
-            foreach ($tasks as $taskID => $task) {
-                if (!empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-
-        case 2:
-            foreach ($tasks as $taskID => $task) {
-                if (empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-        }
-
         /* Sort the array if we have a sort function defined for this
          * field. */
         if (isset($sort_functions[$sortby])) {
@@ -202,6 +221,33 @@ class Nag {
         return $storage->get($task);
     }
 
+    /**
+     * Returns the number of taks in task lists that the current user owns.
+     *
+     * @return integer  The number of tasks that the user owns.
+     */
+    function countTasks()
+    {
+        static $count;
+        if (isset($count)) {
+            return $count;
+        }
+
+        $tasklists = Nag::listTasklists(true, PERMS_ALL);
+
+        $count = 0;
+        foreach (array_keys($tasklists) as $tasklist) {
+            /* Create a Nag storage instance. */
+            $storage = &Nag_Driver::singleton($tasklist);
+            $storage->retrieve();
+
+            /* Retrieve the task list from storage. */
+            $count += count($storage->listTasks());
+        }
+
+        return $count;
+    }
+
     /**
      * Returns all the alarms active right on $date.
      *
@@ -393,18 +439,50 @@ class Nag {
         return $alarm_text;
     }
 
+    /**
+     * Returns the specified permission for the current user.
+     *
+     * @since Nag 2.1
+     *
+     * @param string $permission  A permission, currently only 'max_tasks'.
+     *
+     * @return mixed  The value of the specified permission.
+     */
+    function hasPermission($permission)
+    {
+        global $perms;
+
+        if (!$perms->exists('nag:' . $permission)) {
+            return true;
+        }
+
+        $allowed = $perms->getPermissions('nag:' . $permission);
+        if (is_array($allowed)) {
+            switch ($permission) {
+            case 'max_tasks':
+                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
+                break;
+            }
+        }
+
+        return $allowed;
+    }
+
     /**
      * Build Nag's list of menu items.
      */
     function getMenu($returnType = 'object')
     {
-        global $conf, $registry, $browser, $print_link;
+        global $conf, $registry, $print_link;
 
         require_once 'Horde/Menu.php';
 
         $menu = &new Menu();
         $menu->add(Horde::applicationUrl('list.php'), _("_List Tasks"), 'nag.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
-        if (Nag::getDefaultTasklist(PERMS_EDIT)) {
+        if (Nag::getDefaultTasklist(PERMS_EDIT) &&
+            (!empty($conf['hooks']['permsdenied']) ||
+             Nag::hasPermission('max_tasks') === true ||
+             Nag::hasPermission('max_tasks') > Nag::countTasks())) {
             $menu->add(Horde::applicationUrl(Util::addParameter('task.php', 'actionID', 'add_task')), _("_New Task"), 'add.png', null, null, null, Util::getFormData('task') ? '__noselection' : null);
         }
         $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
@@ -419,8 +497,8 @@ class Nag {
         }
 
         /* Print. */
-        if ($conf['menu']['print'] && isset($print_link) && $browser->hasFeature('javascript')) {
-            $menu->add('#', _("_Print"), 'print.png', $registry->getImageDir('horde'), '', "popup('$print_link'); return false;");
+        if ($conf['menu']['print'] && isset($print_link)) {
+            $menu->add($print_link, _("_Print"), 'print.png', $registry->getImageDir('horde'), '_blank', 'popup(this.href); return false;');
         }
 
         if ($returnType == 'object') {
@@ -472,6 +550,171 @@ class Nag {
         $notification->notify(array('listeners' => 'status'));
     }
 
+    /**
+     * Sends email notifications that a task has been added, edited, or
+     * deleted to users that want such notifications.
+     *
+     * @param string $action     The event action. One of "add", "edit", or
+     *                           "delete".
+     * @param string $tasklist   The tasklist of the task we are dealing with.
+     * @param string $name       The name (short) of the task.
+     * @param string $desc       The description (long) of the task.
+     * @param integer $due       The due date of the task.
+     * @param integer $priority  The priority of the task.
+     */
+    function sendNotification($action, $tasklist, $name, $desc, $due, $priority)
+    {
+        global $conf;
+
+        switch ($action) {
+        case 'add':
+            $subject = _("Task added:");
+            $notification_message = _("You requested to be notified when tasks are added to your tasklists.") . "\n\n" . _("The task \"%s\" has been added to \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        case 'edit':
+            $subject = _("Task modified:");
+            $notification_message = _("You requested to be notified when tasks are edited on your tasklists.") . "\n\n" . _("The task \"%s\" has been edited on \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        case 'delete':
+            $subject = _("Task deleted:");
+            $notification_message = _("You requested to be notified when tasks are deleted from your tasklists.") . "\n\n" . _("The task \"%s\" has been deleted from \"%s\" tasklist, with a due date of: %s.");
+            break;
+
+        default:
+            return PEAR::raiseError('Unknown event action: ' . $action);
+        }
+
+        require_once 'Horde/Group.php';
+        require_once 'Horde/Identity.php';
+        require_once 'Horde/MIME.php';
+        require_once 'Horde/MIME/Headers.php';
+        require_once 'Horde/MIME/Message.php';
+
+        $share = &$GLOBALS['nag_shares']->getShare($tasklist);
+        if (is_a($share, 'PEAR_Error')) {
+            return $share;
+        }
+
+        $groups = &Group::singleton();
+        $recipients = array();
+        $identity = &Identity::singleton();
+        $from = $identity->getDefaultFromAddress(true);
+
+        $owner = $share->get('owner');
+        if (Nag::_notificationPref($owner, 'owner')) {
+            $recipients[$owner] = true;
+        }
+
+        foreach ($share->listUsers(PERMS_READ) as $user) {
+            if (!isset($recipients[$user])) {
+                $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+            }
+        }
+        foreach ($share->listGroups(PERMS_READ) as $group) {
+            $group = $groups->getGroupById($group);
+            if (is_a($group, 'PEAR_Error')) {
+                continue;
+            }
+            $group_users = $group->listAllUsers();
+            if (is_a($group_users, 'PEAR_Error')) {
+                Horde::logMessage($group_users, __FILE__, __LINE__, PEAR_LOG_ERR);
+                continue;
+            }
+            foreach ($group_users as $user) {
+                if (!isset($recipients[$user])) {
+                    $recipients[$user] = Nag::_notificationPref($user, 'read', $tasklist);
+                }
+            }
+        }
+
+        $addresses = array();
+        foreach ($recipients as $user => $send) {
+            if ($send) {
+                $identity = &Identity::singleton('none', $user);
+                $email = $identity->getValue('from_addr');
+                if (strstr($email, '@')) {
+                    list($mailbox, $host) = explode('@', $email);
+                    $addresses[] = MIME::rfc822WriteAddress($mailbox, $host, $identity->getValue('fullname'));
+                }
+            }
+        }
+
+        if (!count($addresses)) {
+            return;
+        }
+
+        $msg_headers = &new MIME_Headers();
+        $msg_headers->addMessageIdHeader();
+        $msg_headers->addAgentHeader();
+        $msg_headers->addHeader('Date', date('r'));
+        $msg_headers->addHeader('From', $from);
+        $msg_headers->addHeader('Subject', $subject . ' ' . $name);
+
+        $message = "\n" . sprintf($notification_message, $name, $share->get('name'), $due ? strftime('%x %X', $due) : 'no due date' . "\n\n" . $desc);
+
+        $mime = &new MIME_Message();
+        $body = &new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+
+        $mime->addPart($body);
+        $msg_headers->addMIMEHeaders($mime);
+
+        $mail_driver = $conf['mailer']['type'];
+        $mail_params = $conf['mailer']['params'];
+        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
+            empty($mail_params['username'])) {
+            $mail_params['username'] = Auth::getAuth();
+            $mail_params['password'] = Auth::getCredential('password');
+        }
+
+        Horde::logMessage(sprintf('Sending event notifications for %s to %s', $name, implode(', ', $addresses)), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        return $mime->send(implode(', ', $addresses), $msg_headers, $mail_driver, $mail_params);
+    }
+
+    /**
+     * Returns whether a user wants email notifications for a tasklist.
+     *
+     * @access private
+     *
+     * @todo This method is causing a memory leak somewhere, noticeable if
+     *       importing a large amount of events.
+     *
+     * @param string $user      A user name.
+     * @param string $mode      The check "mode". If "owner", the method checks
+     *                          if the user wants notifications only for
+     *                          tasklists he owns. If "read", the method checks
+     *                          if the user wants notifications for all
+     *                          tasklists he has read access to, or only for
+     *                          shown tasklists and the specified tasklist is
+     *                          currently shown.
+     * @param string $tasklist  The name of the tasklist if mode is "read".
+     *
+     * @return boolean  True if the user wants notifications for the tasklist.
+     */
+    function _notificationPref($user, $mode, $tasklist = null)
+    {
+        $prefs = &Prefs::singleton($GLOBALS['conf']['prefs']['driver'],
+                                   'nag', $user, '', null,
+                                   false);
+        $prefs->retrieve();
+
+        $notification = $prefs->getValue('task_notification');
+        switch ($notification) {
+        case 'owner':
+            return $mode == 'owner';
+        case 'read':
+            return $mode == 'read';
+        case 'show':
+            if ($mode == 'read') {
+                $display_tasklists = unserialize($prefs->getValue('display_tasklists'));
+                return in_array($tasklist, $display_tasklists);
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Comparison function for sorting tasks by priority.
      *
@@ -483,7 +726,9 @@ class Nag {
      */
     function _sortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) return 0;
+        if ($a['priority'] == $b['priority']) {
+            return 0;
+        }
         return ($a['priority'] > $b['priority']) ? 1 : -1;
     }
 
@@ -498,7 +743,9 @@ class Nag {
      */
     function _rsortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) return 0;
+        if ($a['priority'] == $b['priority']) {
+            return 0;
+        }
         return ($a['priority'] > $b['priority']) ? -1 : 1;
     }
 
@@ -541,7 +788,8 @@ class Nag {
      */
     function _sortByCategory($a, $b)
     {
-        return strcasecmp($a['category'], $b['category']);
+        return strcasecmp($a['category'] ? $a['category'] : _("Unfiled"),
+                          $b['category'] ? $b['category'] : _("Unfiled"));
     }
 
     /**
@@ -555,7 +803,8 @@ class Nag {
      */
     function _rsortByCategory($a, $b)
     {
-        return strcasecmp($b['category'], $a['category']);
+        return strcasecmp($b['category'] ? $b['category'] : _("Unfiled"),
+                          $a['category'] ? $a['category'] : _("Unfiled"));
     }
 
     /**
@@ -621,7 +870,9 @@ class Nag {
      */
     function _sortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) return 0;
+        if ($a['completed'] == $b['completed']) {
+            return 0;
+        }
         return ($a['completed'] > $b['completed']) ? -1 : 1;
     }
 
@@ -636,7 +887,9 @@ class Nag {
      */
     function _rsortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) return 0;
+        if ($a['completed'] == $b['completed']) {
+            return 0;
+        }
         return ($a['completed'] < $b['completed']) ? -1 : 1;
     }
 
