@@ -2,21 +2,26 @@
 /**
  * Nag external API interface.
  *
- * $Horde: nag/lib/api.php,v 1.100.10.6 2005/10/05 13:59:30 jan Exp $
+ * $Horde: nag/lib/api.php,v 1.100.10.16.2.3 2008/01/09 22:00:57 chuck Exp $
  *
- * This file defines Nag's external API interface. Other applications
- * can interact with Nag through this API.
+ * This file defines Nag's external API interface. Other applications can
+ * interact with Nag through this API.
  *
  * @package Nag
  */
 
+$_services['perms'] = array(
+    'args' => array(),
+    'type' => '{urn:horde}hashHash'
+);
+
 $_services['show'] = array(
     'link' => '%application%/view.php?tasklist=|tasklist|&task=|task|&uid=|uid|',
 );
 
 $_services['browse'] = array(
     'args' => array('path' => 'string'),
-    'type' => '{urn:hash}hashHash',
+    'type' => '{urn:horde}hashHash',
 );
 
 $_services['addTasklist'] = array(
@@ -26,22 +31,27 @@ $_services['addTasklist'] = array(
 
 $_services['listTasklists'] = array(
     'args' => array('owneronly' => 'boolean', 'permission' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['listTasks'] = array(
     'args' => array('sortby' => 'string', 'sortdir' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['list'] = array(
     'args' => array(),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['listBy'] = array(
     'args' => array('action' => 'string', 'timestamp' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
+);
+
+$_services['getActionTimestamp'] = array(
+    'args' => array('uid' => 'string', 'timestamp' => 'int'),
+    'type' => 'int',
 );
 
 $_services['import'] = array(
@@ -50,12 +60,12 @@ $_services['import'] = array(
 );
 
 $_services['export'] = array(
-    'args' => array('uid' => 'string', 'contentType' => '{urn:hash}stringArray'),
+    'args' => array('uid' => 'string', 'contentType' => '{urn:horde}stringArray'),
     'type' => 'string',
 );
 
 $_services['delete'] = array(
-    'args' => array('uid' => '{urn:hash}stringArray'),
+    'args' => array('uid' => '{urn:horde}stringArray'),
     'type' => 'boolean',
 );
 
@@ -65,6 +75,16 @@ $_services['replace'] = array(
 );
 
 
+function _nag_perms()
+{
+    $perms = array();
+    $perms['tree']['nag']['max_tasks'] = false;
+    $perms['title']['nag:max_tasks'] = _("Maximum Number of Tasks");
+    $perms['type']['nag:max_tasks'] = 'int';
+
+    return $perms;
+}
+
 function _nag_listTasks($sortby = null, $sortdir = null)
 {
     require_once dirname(__FILE__) . '/base.php';
@@ -107,6 +127,15 @@ function _nag_addTasklist($name, $description = '')
     return $tasklistId;
 }
 
+function __nag_modified($uid)
+{
+    $modified = _nag_getActionTimestamp($uid, 'modify');
+    if (empty($modified)) {
+        $modified = _nag_getActionTimestamp($uid, 'add');
+    }
+    return $modified;
+}
+
 /**
  * Browse through Nag's object tree.
  *
@@ -118,15 +147,6 @@ function _nag_addTasklist($name, $description = '')
  */
 function _nag_browse($path = '', $properties = array())
 {
-    function _modified($uid)
-    {
-        $modified = _nag_getActionTimestamp($uid, 'modify');
-        if (empty($modified)) {
-            $modified = _nag_getActionTimestamp($uid, 'add');
-        }
-        return $modified;
-    }
-
     require_once dirname(__FILE__) . '/base.php';
     global $registry;
 
@@ -188,7 +208,7 @@ function _nag_browse($path = '', $properties = array())
                 $results[$key]['contentlength'] = strlen($data);
             }
             if (in_array('modified', $properties)) {
-                $results[$key]['modified'] = _modified($task['uid']);
+                $results[$key]['modified'] = __nag_modified($task['uid']);
             }
             if (in_array('created', $properties)) {
                 $results[$key]['created'] = _nag_getActionTimestamp($task['uid'], 'add');
@@ -213,7 +233,7 @@ function _nag_browse($path = '', $properties = array())
 
             $result = array('data' => _nag_export($task['uid'], 'text/x-vtodo'),
                             'mimetype' => 'text/x-vtodo');
-            $modified = _modified($task['uid']);
+            $modified = __nag_modified($task['uid']);
             if (!empty($modified)) {
                 $result['mtime'] = $modified;
             }
@@ -242,9 +262,12 @@ function _nag_listTasklists($owneronly, $permission)
  * Returns an array of UIDs for all tasks that the current user is authorized
  * to see.
  *
- * @return array  An array of UIDs for all tasks the user can access.
+ * @param variant $tasklist  The tasklist or an array of taskslists to list.
+ *
+ * @return array             An array of UIDs for all tasks
+ *                           the user can access.
  */
-function _nag_list()
+function _nag_list($tasklist = null)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $conf;
@@ -254,7 +277,16 @@ function _nag_list()
         return PEAR::raiseError('Not configured');
     }
 
-    $tasks = Nag::listTasks();
+    if ($tasklist === null) {
+        $tasklist = Nag::getDefaultTasklist();
+    }
+
+    if (!array_key_exists($tasklist,
+                          Nag::listTasklists(false, PERMS_READ))) {
+        return PEAR::raiseError(_("Permission Denied"));
+    }
+
+    $tasks = Nag::listTasks(null, null, null, $tasklist);
     if (is_a($tasks, 'PEAR_Error')) {
         return $tasks;
     }
@@ -273,25 +305,66 @@ function _nag_list()
  *
  * @param string  $action     The action to check for - add, modify, or delete.
  * @param integer $timestamp  The time to start the search.
+ * @param string  $tasklist   The tasklist to be used. If 'null', the
+ *                            user's default tasklist will be used.
  *
  * @return array  An array of UIDs matching the action and time criteria.
  */
-function &_nag_listBy($action, $timestamp)
+function _nag_listBy($action, $timestamp, $tasklist = null)
 {
     require_once dirname(__FILE__) . '/base.php';
-    require_once 'Horde/History.php';
+
+    if ($tasklist === null) {
+        $tasklist = Nag::getDefaultTasklist();
+    }
+
+    if (!array_key_exists($tasklist,
+                          Nag::listTasklists(false, PERMS_READ))) {
+        return PEAR::raiseError(_("Permission Denied"));
+    }
 
     $history = &Horde_History::singleton();
-    $histories = $history->getByTimestamp('>', $timestamp, array(array('op' => '=', 'field' => 'action', 'value' => $action)), 'nag');
+    $histories = $history->getByTimestamp('>', $timestamp, array(array('op' => '=', 'field' => 'action', 'value' => $action)), 'nag:' . $tasklist);
     if (is_a($histories, 'PEAR_Error')) {
         return $histories;
     }
 
-    return array_keys($histories);
+    // Strip leading nag:username:.
+    return preg_replace('/^([^:]*:){2}/', '', array_keys($histories));
 }
 
 /**
- * Imports a task represented in the specified content type.
+ * Returns the timestamp of an operation for a given uid an action.
+ *
+ * @param string $uid      The uid to look for.
+ * @param string $action   The action to check for - add, modify, or delete.
+ * @param string $tasklist The tasklist to be used. If 'null', the
+ *                         user's default tasklist will be used.
+ *
+ * @return integer  The timestamp for this action.
+ */
+function _nag_getActionTimestamp($uid, $action, $tasklist = null)
+{
+    require_once dirname(__FILE__) . '/base.php';
+
+    if ($tasklist === null) {
+        $tasklist = Nag::getDefaultTasklist();
+    }
+
+    if (!array_key_exists($tasklist,
+                          Nag::listTasklists(false, PERMS_READ))) {
+        return PEAR::raiseError(_("Permission Denied"));
+    }
+
+    $history = &Horde_History::singleton();
+    return $history->getActionTimestamp('nag:' . $tasklist . ':' . $uid, $action);
+}
+
+/**
+ * Imports one or more tasks represented in the specified content type.
+ *
+ * If a UID is present in the content and the task is already in the
+ * database, a replace is performed rather than an add.
  *
  * @param string $content      The content of the task.
  * @param string $contentType  What format is the data in? Currently supports:
@@ -302,11 +375,13 @@ function &_nag_listBy($action, $timestamp)
  *                             imported.  If 'null', the user's default
  *                             tasklist will be used.
  *
- * @return string  The new UID, or PEAR_Error on failure.
+ * @return string  The new UID on one import, an array of UIDs on multiple imports,
+ *                 or PEAR_Error on failure.
  */
 function _nag_import($content, $contentType, $tasklist = null)
 {
     require_once dirname(__FILE__) . '/base.php';
+
     global $prefs;
 
     if ($tasklist === null) {
@@ -317,93 +392,96 @@ function _nag_import($content, $contentType, $tasklist = null)
         return PEAR::raiseError(_("Permission Denied"));
     }
 
+    require_once 'Horde/iCalendar.php';
+
     /* Create a Nag_Driver instance. */
     require_once NAG_BASE . '/lib/Driver.php';
+
     $storage = &Nag_Driver::singleton($tasklist);
 
     switch ($contentType) {
     case 'text/x-icalendar':
     case 'text/x-vcalendar':
     case 'text/x-vtodo':
+    case 'text/calendar':
+        $iCal = &new Horde_iCalendar();
         if (!is_a($content, 'Horde_iCalendar_vtodo')) {
-            require_once 'Horde/iCalendar.php';
-            $iCal = &new Horde_iCalendar();
             if (!$iCal->parsevCalendar($content)) {
                 return PEAR::raiseError(_("There was an error importing the iCalendar data."));
             }
-
-            $components = $iCal->getComponents();
-            switch (count($components)) {
-            case 0:
-                return PEAR::raiseError(_("No iCalendar data was found."));
-
-            case 1:
-                $content = $components[0];
-                break;
-
-            default:
-                $ids = array();
-                foreach ($components as $content) {
-                    if (is_a($content, 'Horde_iCalendar_vtodo')) {
-                        $task = $storage->fromiCalendar($content);
-                        $taskId = $storage->add(
-                            isset($task['name']) ? $task['name'] : '',
-                            isset($task['desc']) ? $task['desc'] : '',
-                            isset($task['due']) ? $task['due'] : '',
-                            isset($task['priority']) ? $task['priority'] : 3,
-                            !empty($task['completed']) ? 1 : 0,
-                            isset($task['category']) ? $task['category'] : '',
-                            isset($task['alarm']) ? $task['alarm'] : 0,
-                            isset($task['uid']) ? $task['uid'] : null);
-                        if (is_a($taskId, 'PEAR_Error')) {
-                            return $taskId;
-                        }
-                        $ids[] = $taskId;
-                    }
-                }
-                return $ids;
-            }
+        } else {
+            $iCal->addComponent($content);
         }
 
-        /**
-         * @TODO Need to check for alarms.
-         */
-        $task = $storage->fromiCalendar($content);
-        $taskId = $storage->add(isset($task['name']) ? $task['name'] : '',
-                                isset($task['desc']) ? $task['desc'] : '',
-                                isset($task['due']) ? $task['due'] : '',
-                                isset($task['priority']) ? $task['priority'] : 3,
-                                !empty($task['completed']) ? 1 : 0,
-                                isset($task['category']) ? $task['category'] : '',
-                                0, // Alarms are missing.
-                                isset($task['uid']) ? $task['uid'] : null);
-        break;
+        $components = $iCal->getComponents();
+        if (count($components) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        }
 
-    default:
-        return PEAR::raiseError(_("Unsupported Content-Type."));
+        $ids = array();
+        foreach ($components as $content) {
+            if (is_a($content, 'Horde_iCalendar_vtodo')) {
+                $task = $storage->fromiCalendar($content);
+                if (isset($task['uid']) && !is_a(($existing = $storage->getByUID($task['uid'])), 'PEAR_Error')) {
+                    $taskId = $existing['task_id'];
+                    $result = $storage->modify($taskId,
+                        isset($task['name']) ? $task['name'] : $existing['name'],
+                        isset($task['desc']) ? $task['desc'] : $existing['desc'],
+                        isset($task['due']) ? $task['due'] : $existing['due'],
+                        isset($task['priority']) ? $task['priority'] : $existing['priority'],
+                        isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
+                        isset($task['category']) ? $task['category'] : $existing['category'],
+                        isset($task['alarm']) ? $task['alarm'] : $existing['alarm']
+                        );
+
+                    if (is_a($result, 'PEAR_Error')) {
+                        return $result;
+                    }
+                    $ids[] = $task['uid'];
+                } else {
+                    $newTask = $storage->add(
+                        isset($task['name']) ? $task['name'] : '',
+                        isset($task['desc']) ? $task['desc'] : '',
+                        isset($task['due']) ? $task['due'] : 0,
+                        isset($task['priority']) ? $task['priority'] : 3,
+                        !empty($task['completed']) ? 1 : 0,
+                        isset($task['category']) ? $task['category'] : '',
+                        isset($task['alarm']) ? $task['alarm'] : 0,
+                        isset($task['uid']) ? $task['uid'] : null
+                        );
+
+                    if (is_a($newTask, 'PEAR_Error')) {
+                        return $newTask;
+                    }
+                    $ids[] = $newTask[1]; // use UID rather than ID
+                }
+
+            }
+        }
+        if (count($ids) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        } else if (count($ids) == 1) {
+            return $ids[0];
+        }
+        return $ids;
     }
 
-    if (is_a($taskId, 'PEAR_Error')) {
-        return $taskId;
-    }
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
 
-    $task = $storage->get($taskId);
-    return $task['uid'];
 }
 
 /**
  * Exports a task, identified by UID, in the requested content type.
  *
- * @param string $uid          Identify the task to export.
- * @param mixed  $contentType  What format should the data be in?
- *                             Either a string with one of:
- *                             text/x-vtodo
- *                             text/x-vcalendar
+ * @param string $uid         Identify the task to export.
+ * @param string $contentType  What format should the data be in?
+ *                            A string with one of:
+ *                            <pre>
+ *                             text/calendar (VCALENDAR 2.0. Recommended as this is specified in rfc2445)
+ *                             text/x-vtodo (seems to be used by horde only. Do we need this?)
+ *                             text/x-vcalendar (old VCALENDAR 1.0 format. Still in wide use)
  *                             text/x-icalendar
- *                             'ContentType':  as above
- *                             'ENCODING': (optional) character encoding
- *                                         for strings fields
- *                             'CHARSET':  (optional) charset. Like UTF-8
+ *                            </pre>
  *
  * @return string  The requested data.
  */
@@ -417,44 +495,35 @@ function _nag_export($uid, $contentType)
         return $task;
     }
 
-    if (!array_key_exists($task['tasklist_id'], Nag::listTasklists(false, PERMS_EDIT))) {
+    if (!array_key_exists($task['tasklist_id'], Nag::listTasklists(false, PERMS_READ))) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
-    if (is_array($contentType)) {
-        $options = $contentType;
-        $contentType = $options['ContentType'];
-        unset($options['ContentType']);
-    } else {
-        $options = array();
-    }
-
+    $version = '2.0';
     switch ($contentType) {
-    case 'text/x-icalendar':
     case 'text/x-vcalendar':
+        $version = '1.0';
+    case 'text/calendar':
+    case 'text/x-icalendar':
     case 'text/x-vtodo':
         require_once dirname(__FILE__) . '/version.php';
         require_once 'Horde/iCalendar.php';
 
         // Create the new iCalendar container.
-        $iCal = &new Horde_iCalendar();
-        $iCal->setAttribute('VERSION', '2.0');
+        $iCal = &new Horde_iCalendar($version);
         $iCal->setAttribute('PRODID', '-//The Horde Project//Nag ' . NAG_VERSION . '//EN');
         $iCal->setAttribute('METHOD', 'PUBLISH');
 
         // Create new vTodo object.
         $vTodo = $storage->toiCalendar($task, $iCal);
-
-        // Set encoding options for all string values.
-        $vTodo->setParameter('SUMMARY', $options);
-        $vTodo->setParameter('DESCRIPTION', $options);
+        $vTodo->setAttribute('VERSION', $version);
 
         $iCal->addComponent($vTodo);
 
         return $iCal->exportvCalendar();
 
     default:
-        return PEAR::raiseError(_("Unsupported Content-Type."));
+        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
     }
 }
 
@@ -500,12 +569,18 @@ function _nag_delete($uid)
  * Replaces the task identified by UID with the content represented in the
  * specified content type.
  *
+ * If you want to replace multiple tasks with the UID specified in the
+ * VCALENDAR data, you may use _nag_import instead. This automatically
+ * does a replace if existings UIDs are found.
+ *
+ *
  * @param string $uid          Identify the task to replace.
  * @param string $content      The content of the task.
  * @param string $contentType  What format is the data in? Currently supports:
  *                             text/x-icalendar
  *                             text/x-vcalendar
  *                             text/x-vtodo
+ *                             text/calendar
  *
  * @return boolean  Success or failure.
  */
@@ -514,17 +589,18 @@ function _nag_replace($uid, $content, $contentType)
     require_once dirname(__FILE__) . '/base.php';
 
     $storage = &Nag_Driver::singleton();
-    $task = $storage->getByUID($uid);
-    if (is_a($task, 'PEAR_Error')) {
-        return $task;
+    $existing = $storage->getByUID($uid);
+    if (is_a($existing, 'PEAR_Error')) {
+        return $existing;
     }
-    $taskId = $task['task_id'];
+    $taskId = $existing['task_id'];
 
-    if (!array_key_exists($task['tasklist_id'], Nag::listTasklists(false, PERMS_EDIT))) {
+    if (!array_key_exists($existing['tasklist_id'], Nag::listTasklists(false, PERMS_EDIT))) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
     switch ($contentType) {
+    case 'text/calendar':
     case 'text/x-icalendar':
     case 'text/x-vcalendar':
     case 'text/x-vtodo':
@@ -549,23 +625,21 @@ function _nag_replace($uid, $content, $contentType)
             }
         }
 
-        /**
-         * @TODO Need to check for alarms.
-         */
         $task = $storage->fromiCalendar($content);
         $result = $storage->modify($taskId,
-                                   isset($task['name']) ? $task['name'] : '',
-                                   isset($task['desc']) ? $task['desc'] : '',
-                                   isset($task['due']) ? $task['due'] : '',
-                                   isset($task['priority']) ? $task['priority'] : 3,
-                                   !empty($task['completed']) ? 1 : 0,
-                                   isset($task['category']) ? $task['category'] : '',
-                                   0 // Alarms are missing.
-                                   );
+                                    isset($task['name']) ? $task['name'] : $existing['name'],
+                                    isset($task['desc']) ? $task['desc'] : $existing['desc'],
+                                    isset($task['due']) ? $task['due'] : $existing['due'],
+                                    isset($task['priority']) ? $task['priority'] : $existing['priority'],
+                                    isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
+                                    isset($task['category']) ? $task['category'] : $existing['category'],
+                                    isset($task['alarm']) ? $task['alarm'] : $existing['alarm']
+                                    );
+
         break;
 
     default:
-        return PEAR::raiseError(_("Unsupported Content-Type."));
+        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
     }
 
     return $result;
