@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: nag/list.php,v 1.93.8.4 2006/01/01 21:29:06 jan Exp $
+ * $Horde: nag/list.php,v 1.93.8.6 2007/01/02 13:55:12 jan Exp $
  *
- * Copyright 2001-2006 Jon Parise <jon@horde.org>
+ * Copyright 2001-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL).  If you did
  * not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -12,7 +12,7 @@
 require_once NAG_BASE . '/lib/base.php';
 require_once 'Horde/Variables.php';
 
-$vars = &Variables::getDefaultVariables();
+$vars = Variables::getDefaultVariables();
 
 /* Get the current action ID. */
 $actionID = Util::getFormData('actionID');
@@ -68,7 +68,7 @@ $print_view = (bool)Util::getFormData('print');
 if (!$print_view) {
     Horde::addScriptFile('popup.js', 'horde', true);
     Horde::addScriptFile('tooltip.js', 'horde', true);
-    Horde::addScriptFile('httpclient.js', 'horde', true);
+    Horde::addScriptFile('prototype.js', 'nag', true);
     Horde::addScriptFile('tables.js', 'nag', true);
     $print_link = Horde::applicationUrl(Util::addParameter('list.php', array('print' => 1)));
 }
@@ -83,7 +83,7 @@ if ($print_view) {
     if (!$prefs->isLocked('show_completed')) {
     	require_once 'Horde/UI/Tabs.php';
     	$listurl = Horde::applicationUrl('list.php');
-    	$tabs = &new Horde_UI_Tabs('show_completed', $vars);
+    	$tabs = new Horde_UI_Tabs('show_completed', $vars);
     	$tabs->addTab(_("_All tasks"), $listurl, 1);
     	$tabs->addTab(_("Incom_plete tasks"), $listurl, 0);
     	$tabs->addTab(_("_Completed tasks"), $listurl, 2);
