@@ -1,9 +1,9 @@
--- $Horde: nag/scripts/sql/nag.sql,v 1.4 2004/11/16 20:43:52 chuck Exp $
+-- $Horde: nag/scripts/sql/nag.sql,v 1.4.8.2 2007/09/27 22:41:03 jan Exp $
 
 CREATE TABLE nag_tasks (
     task_id         VARCHAR(32) NOT NULL,
     task_owner      VARCHAR(255) NOT NULL,
-    task_name       VARCHAR(64) NOT NULL,
+    task_name       VARCHAR(255) NOT NULL,
     task_uid        VARCHAR(255) NOT NULL,
     task_desc       TEXT,
     task_due        INT,
@@ -18,5 +18,3 @@ CREATE TABLE nag_tasks (
 
 CREATE INDEX nag_tasklist_idx ON nag_tasks (task_owner);
 CREATE INDEX nag_uid_idx ON nag_tasks (task_uid);
-
-GRANT SELECT, INSERT, UPDATE, DELETE ON nag_tasks TO horde;
